/**
 * Copyright (c) 2001/2003 Distributed Systems Research Group.
 * All rights reserved.
 *
 * For more information on Distributed Systems Research Group (DSRG),
 * please see <http://www.ics.agh.edu.pl/>.
 *
 * @author radzisz
 * file name: NoteClient.java
 *
 */
package edu.agh.dsrg.rozproszone;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.rmi.NoSuchObjectException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

public class NoteBoardClient implements NoteBoardListener {

	JFrame frame = new JFrame("NoteClient");
	JTextArea textArea = new JTextArea(30, 60);
	JButton bClear = new JButton("clear");
	JButton bAppend = new JButton("append");
	JButton bLogin = new JButton("login");
	JButton bLogout = new JButton("logout");

	JTextField userName = new JTextField(10);
	JTextField userNick = new JTextField(10);
	JTextField userLastName = new JTextField(10);
	JTextField textField = new JTextField(50);

	NoteBoard board;
	String host;
	int port;
	User user;

	NoteBoardClient(String host, int port) {
		this.host = host;
		this.port = port;

		bClear.addActionListener(new ActionHandler());
		bLogin.addActionListener(new ActionHandler());
		bLogout.addActionListener(new ActionHandler());
		bAppend.addActionListener(new ActionHandler());

		frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		JScrollPane scrollPane =
			new JScrollPane(
				textArea,
				JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		textArea.setEditable(false);
		frame.getContentPane().add(scrollPane, BorderLayout.CENTER);

		JPanel bottomPanel = new JPanel();
		bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.Y_AXIS));

		JPanel userPanel = new JPanel();
		userPanel.add(new JLabel("nick"));
		userPanel.add(userNick);
		userPanel.add(new JLabel("name"));
		userPanel.add(userName);
		userPanel.add(new JLabel("lastname"));
		userPanel.add(userLastName);
		userPanel.add(bLogin);
		userPanel.add(bLogout);
		bLogout.disable();

		JPanel textPanel = new JPanel();
		textPanel.add(new JLabel("text"));
		textPanel.add(textField);
		textPanel.add(bAppend);

		JPanel buttonPanel = new JPanel();
		buttonPanel.add(bClear);

		bottomPanel.add(userPanel);
		bottomPanel.add(textPanel);
		bottomPanel.add(buttonPanel);

		frame.getContentPane().add(bottomPanel, BorderLayout.SOUTH);
		frame.pack();
		frame.setVisible(true);

		//init conenction with remote noteBoard
		initRemote();

		//make myself remotely accessible
		//to do  - export this object

	}
	class ActionHandler extends WindowAdapter implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == bClear)
				textArea.setText("");

			if (e.getSource() == bAppend) {
				if (user == null) {
					log("user is null!!!");
					return;
				}
				if (board == null) {
					log("not connected to board!!!");
					return;
				}
				//to do
				//call the appendText on the remote board

			}

			if (e.getSource() == bLogin) {
				if (board == null) {
					log("not connected to board!!!");
					return;
				}
			  //to do
			  //create User instance

			  //register in board
			  //by calling register() on board interface
			   bLogin.setEnabled(false);
			   bLogout.setEnabled(true);
			}

			if (e.getSource() == bLogout) {
			  //to do
			  //unregister from board
			  //by calling unregister() on board interface
			   bLogin.setEnabled(true);
			   bLogout.setEnabled(false);
			}

		}

		public void windowClosing(WindowEvent e) {
				//to do
				//logout - may throw exception
				//and make me not accessible remotely
			//disconnect remote object
			board = null;
			System.exit(0);
		}
	}

	private void initRemote() {
		try {
			Registry registry = LocateRegistry.getRegistry(host, port);
			log("registry set to:" + host + ":" + port);
			String url="NoteBoard";
			log("binding to: " + url);
			//lookup the object
			Object obj = registry.lookup(url);

			//or
			//String url = "//" + host + ":" + port + "/" + "NoteBoard";
			//Object obj = Naming.bind(url)

			log("get object");
			board = (NoteBoard) obj;
		} catch (RemoteException e) {
			displayError(e.toString());
			e.printStackTrace();
		} catch (NotBoundException e) {
			displayError(e.toString());
			e.printStackTrace();
		}

	}

	/* (non-Javadoc)
	 * @see edu.agh.dsrg.rozproszone.NoteBoardListener#onNewText(java.lang.String)
	 */
	public void onNewText(String text) {
		textArea.append(text);
	}

	private void log(String text) {
		textArea.append("local:" + text + "\n");
	}

	public static void main(String[] args) {
		if (args.length == 2)
			try {
				new NoteBoardClient(args[0], Integer.parseInt(args[1]));
				return;
			} catch (Exception e) {
			}
		usage();
	}

	private static void usage() {
		System.out.println("Usage: program <rmiDhost> <rmiDport>");
	}

	private void displayError(String text) {
		JOptionPane.showInternalMessageDialog(
			frame.getContentPane(),
			text,
			"error",
			JOptionPane.INFORMATION_MESSAGE);
	}

}
