/**
 * Copyright (c) 2001/2003 Distributed Systems Research Group.
 * All rights reserved.
 *
 * For more information on Distributed Systems Research Group (DSRG),
 * please see <http://www.ics.agh.edu.pl/>.
 *
 * @author radzisz
 * file name: BoardServer.java
 *
 */
package edu.agh.dsrg.rozproszone;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.Date;

public class NoteBoardServer {

	public static void main(String[] args) {
		if (args.length == 1) {
			new NoteBoardServer(Integer.parseInt(args[0]));
			return;
		}
		usage();
	}

	NoteBoardServer(int port) {
		log("starting the server");
		try {
			//security manager
			//if (System.getSecurityManager() == null) {
			//	System.setSecurityManager(new RMISecurityManager());
			//}

			//create object instance
			NoteBoardImpl noteBoard = new NoteBoardImpl();
			log("NoteBoard object created");

			//export object
			UnicastRemoteObject.exportObject(noteBoard);
			log("NoteBoard object exported");

			/*
			   The Naming class provides methods for storing and obtaining references to remote objects
			   in a remote object registry. Each method of the Naming class takes as one of its arguments
			   a name that is a java.lang.String in URL format (without the scheme component) of the form:

									//host:port/name
					 where host is the host (remote or local) where the registry is located, port is the port
					 number on which the registry accepts calls, and where name is a simple string uninterpreted
					 by the registry. Both host and port are optional. If host is omitted, the host defaults to
					 the local host. If port is omitted, then the port defaults to 1099, the "well-known"
					 port that RMI's registry, rmiregistry, uses.
			*/

			String url = null; //to do - change here

			//to do
			//create local registry on the port specified
			//bind in this registry

			log("Object successfully bind to: " + url);
			log("Server successfully started at: " + new Date());

		} catch (RemoteException e) {
			log("Could not externalize noteboard - cannot continue");
			log(e.toString());
		}

	}

	public static void log(String msg) {
		System.out.println(msg);
	}

	private static void usage() {
		System.out.println("Usage: program <rmiDport>");
	}

}
